@@ -1,4 +1,4 @@
-__version__ = "1.1.3"
+__version__ = "2.0.0"
 
 import logging
 
@@ -14,7 +14,7 @@ try:
     from . import auth
     from .auth import AuthHandler
 except Exception as ex:
-    logging.error(f"Unable to load auth: {ex}")
+    logging.warning("Unable to load auth, please install `functown[jwt]`")
 
 try:
     from . import serialization
@@ -28,7 +28,7 @@ try:
         InsightsMetrics,
         InsightsEvents,
         InsightsTracer,
-        Insights,
+        insights as Insights,  # avoid name clash with insights package
     )
 except Exception as ex:
-    logging.error(f"Unable to load metrics: {ex}")
+    logging.warning("Unable to load metrics, please install `functown[insights]`")
