@@ -8,6 +8,7 @@ from typing import Any, Dict, Tuple, Union
 
 from azure.functions import HttpResponse, HttpRequest
 
+from functown.args import ContentTypes
 from functown.utils import BaseDecorator
 
 
@@ -62,6 +63,8 @@ class SerializationDecorator(BaseDecorator):
 
         # serialize result
         data, mtype = self.serialize(req, res, *args, **kwargs)
+        if isinstance(mtype, ContentTypes):
+            mtype = mtype.value
         return HttpResponse(
             data, status_code=self._code, headers=self._headers, mimetype=mtype
         )
@@ -74,8 +77,9 @@ class DeserializationDecorator(BaseDecorator):
     deserialized request body.
     """
 
-    def __init__(self, func=None, **kwargs):
-        super().__init__(func, ["body"], **kwargs)
+    def __init__(self, func=None, keyword="body", **kwargs):
+        super().__init__(func, [keyword], **kwargs)
+        self._kw = keyword
 
     @abstractmethod
     def deserialize(self, req: HttpRequest, *args, **kwargs) -> Any:
@@ -99,7 +103,7 @@ class DeserializationDecorator(BaseDecorator):
 
         # deserialize request body
         body = self.deserialize(req, *args, **kwargs)
-        kwargs["body"] = body
+        kwargs[self._kw] = body
 
         # execute inner function
         return func(req=req, *args, **kwargs)
